from enum import Enum
from typing import List, Optional

import yaml
from pydantic import BaseModel, parse_obj_as


class ModuleAccessPolicy(Enum):
    GRANT = "grant"
    DENY = "deny"


class ModuleAccessConfiguration(BaseModel):
    pattern: str
    policy: ModuleAccessPolicy
    forbidden: Optional[List[str]]
    authorized: Optional[List[str]]


class Configuration(BaseModel):
    linted_folders: List[str]
    protected_modules: List[ModuleAccessConfiguration]

    @staticmethod
    def from_file(filename: str):
        try:
            with open(filename, "r", encoding="utf-8") as config_file:
                config = yaml.safe_load(config_file)
        except FileNotFoundError:
            print(f"Config file not found in {filename}")
            exit(1)
        return parse_obj_as(Configuration, config)



import os
from typing import Iterable, List

from pydantic import BaseModel


class ModuleInfo(BaseModel):
    module_name: str
    absolute_path: str


class ModulesLoader:
    def __init__(self, linted_folders: List[str]):
        self.linted_folders = linted_folders

    def itermodules(self) -> Iterable:
        for linted_folder in self.linted_folders:
            root_folder_absolute_path = os.path.abspath(linted_folder)
            parent_module_name = root_folder_absolute_path.rsplit("/", maxsplit=1)[1]
            root_folder_absolute_path_size = len(root_folder_absolute_path)

            for parent_folder, _, child_files in os.walk(root_folder_absolute_path):
                for filename in child_files:
                    absolute_path = os.path.join(parent_folder, filename)
                    if filename.endswith(".py"):
                        yield ModuleInfo(
                            absolute_path=absolute_path,
                            module_name=parent_module_name
                            + absolute_path[root_folder_absolute_path_size:-3].replace(
                                "/", "."
                            ),
                        )



from dataclasses import dataclass, field
import os
from typing import List

from tlint.configuration import Configuration
from tlint.linters.forbidden_imports import ForbiddenImportsLinter
from tlint.linters.linter_interface import ILinter
from tlint.loader import ModuleInfo, ModulesLoader

@dataclass
class LinterChain:
    linters: List[ILinter] = field(default_factory=list)

    def add(self, linter: ILinter):
        self.linters.append(linter)

    def lint_module(self, module_info: ModuleInfo):
        for linter in self.linters:
            linter.lint_module(module_info)


class TLint:
    def __init__(self):
        self.__reload_config()

    def __reload_config(self):
        """Load or reload tlint config from a YAML file.

        If environment variable LINTIT_CONFIG is set, it is used as config file path.
        Otherwise config file is assumed to be located either at tlint.yaml or ~/tlint.yaml
        """
        config_file_path = os.getenv("LINTIT_CONFIG")
        if config_file_path is None:
            config_file_path = os.path.abspath("tlint.yaml")

        self.config = Configuration.from_file(config_file_path)

    def build_linter_from_config(self):
        root_linter = LinterChain()
        if len(self.config.protected_modules) > 0:
            root_linter.add(ForbiddenImportsLinter(self.config.protected_modules))
        return root_linter

    def lint(self):
        linter = self.build_linter_from_config()
        modules_loader = ModulesLoader(self.config.linted_folders)
        for moduleinfo in modules_loader.itermodules():
            linter.lint_module(moduleinfo)



import shutil
import sys
from tlint import TLint

__doc__ = """usage: tlint [init | -h]

Python linter with very few options.
All linting config goes in yaml config file.

- Lint project according to tlint.yaml
    tlint

- Generate default linting config
    tlint init

- Show tlint CLI help
    tlint help
    tlint -h
"""

def main():
    if len(sys.argv) > 1:
        if sys.argv[1] == 'init':
            shutil.copy("tlint/conf/example.tlint.yaml", "tlint.yaml")
        elif sys.argv[1] in ('-h', 'help'):
            print(__doc__)
    else:
        TLint().lint()

if __name__ == "__main__":
    main()



from importlib.metadata import entry_points
import setuptools

setuptools.setup(
    name="tlint",
    description="Linter which behaviour is entirely based on a config file.",
    version="0.0.1",
    packages=setuptools.find_packages(),
    entry_points={
        "console_scripts": [
            "tlint=tlint.__main__:main"
        ]
    }
)
